from alembic import command
import unittest
import json

from app import app, db, migrate
from ki.actions import seed
from ki.models import User


class TestProfileEndpoint(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        app.debug = True
        app.config["TESTING"] = True
        app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:///:memory:"
        self.client = app.test_client()

        with app.app_context():
            config = migrate.get_config()
            command.upgrade(config, "head")

        seed(True)

    def tearDown(self):
        db.drop_all()
        db.engine.dispose()

    def test_update_profile(self):
        login_data = {"username": "peter", "password": "geheim"}
        login_response = self.client.post("/users/login", data=json.dumps(login_data), content_type="application/json")

        self.assertEqual(login_response.status_code, 200)
        self.assertIn("token", login_response.json)

        data = {
            "pronouns": "Monsieur",
            "volunteerwork": "ja",
            "freetext": "Hallo",
            "address": {
                "name": "Peeeda",
                "street": "Bachstraße",
                "house_number": "42x",
                "additional": "oben",
                "postcode": "23232",
                "city": "Travemünde",
                "country": "Deutschland"
            },
            "contacts": [{
                "id": 1,
                "contacttype": {
                    "id": 1,
                    "name": "Matrix"
                },
                "content": "@peeda:wtf-eg.de"
            }, {
                "contacttype": {
                    "name": "Rohrpost"
                },
                "content": "Ausgang 2"
            }],
            "skills": [{
                "id": 1,
                "skill": {
                    "id": 3,
                    "name": "Python"
                },
                "level": 4
            }, {
                "skill": {
                    "name": "Tschunkproduktion"
                },
                "level": 5
            }],
            "languages": [{
                "id": 1,
                "language": {
                    "id": "de",
                    "name": "Deutsch"
                },
                "level": 4
            }, {
                "language": {
                    "id": "es",
                    "name": "Spanisch"
                },
                "level": 2
            }]
        }
        response = self.client.post("/users/1/profile",
                                    data=json.dumps(data),
                                    content_type="application/json",
                                    headers={"Authorization": "Bearer " + login_response.json["token"]})

        self.assertEqual(response.status_code, 200)
        with app.app_context():
            user = User.query.filter(User.id == 1).first()
            profile = user.profile
            self.assertEqual("Monsieur", profile.pronouns)
            self.assertEqual("ja", profile.volunteerwork)
            self.assertEqual("Hallo", profile.freetext)

            address = profile.address
            self.assertEqual(address.name, "Peeeda")
            self.assertEqual(address.street, "Bachstraße")
            self.assertEqual(address.house_number, "42x")
            self.assertEqual(address.additional, "oben")
            self.assertEqual(address.postcode, "23232")
            self.assertEqual(address.city, "Travemünde")
            self.assertEqual(address.country, "Deutschland")

            contacts = profile.contacts
            self.assertEqual(len(contacts), 2)

            first_contact = contacts[0]
            self.assertEqual(first_contact.contacttype.name, "Matrix")
            self.assertEqual(first_contact.content, "@peeda:wtf-eg.de")

            second_contact = contacts[1]
            self.assertEqual(second_contact.contacttype.name, "Rohrpost")
            self.assertEqual(second_contact.content, "Ausgang 2")

            skills = profile.skills
            self.assertEqual(len(skills), 2)

            first_skill = skills[0]
            self.assertEqual(first_skill.skill.id, 3)
            self.assertEqual(first_skill.skill.name, "Python")
            self.assertEqual(first_skill.level, 4)

            second_skill = skills[1]
            self.assertEqual(second_skill.skill.id, 13)
            self.assertEqual(second_skill.skill.name, "Tschunkproduktion")
            self.assertEqual(second_skill.level, 5)

            languages = profile.languages
            self.assertEqual(len(languages), 2)

            first_language = languages[0]
            self.assertEqual(first_language.language_id, "de")
            self.assertEqual(first_language.level, 4)

            second_language = languages[1]
            self.assertEqual(second_language.language_id, "es")
            self.assertEqual(second_language.level, 2)

    def test_get_profile(self):
        login_data = {"username": "peter", "password": "geheim"}
        login_response = self.client.post("/users/login", data=json.dumps(login_data), content_type="application/json")

        self.assertEqual(login_response.status_code, 200)
        self.assertIn("token", login_response.json)

        response = self.client.get("/users/1/profile",
                                   headers={"Authorization": "Bearer " + login_response.json["token"]})

        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            response.json, {
                "profile": {
                    "user_id": 1,
                    "nickname": "peternichtlustig",
                    "pronouns": "Herr Dr. Dr.",
                    "freetext": "Ich mag Kaffee",
                    "volunteerwork": "Gartenverein",
                    "address": {
                        "additional": "Hinterhaus",
                        "city": "Bielefeld",
                        "country": "Deutschland",
                        "house_number": "23i",
                        "id": 1,
                        "name": "Peter Nichtlustig",
                        "postcode": "13337",
                        "profile_id": 1,
                        "street": "Waldweg"
                    },
                    "contacts": [{
                        "id": 1,
                        "profile_id": 1,
                        "contacttype": {
                            "id": 1,
                            "name": "Matrix"
                        },
                        "content": "@peter:wtf-eg.de"
                    }, {
                        "id": 2,
                        "profile_id": 1,
                        "contacttype": {
                            "id": 2,
                            "name": "E-Mail"
                        },
                        "content": "peter@wtf-eg.de"
                    }],
                    "skills": [{
                        "profile_id": 1,
                        "skill": {
                            "id": 1,
                            "name": "PHP",
                            "icon_url": "/skills/1/icon"
                        },
                        "level": 5
                    }, {
                        "profile_id": 1,
                        "skill": {
                            "id": 3,
                            "name": "Python",
                            "icon_url": "/skills/3/icon"
                        },
                        "level": 3
                    }],
                    "languages": [{
                        "profile_id": 1,
                        "language": {
                            "id": "de",
                            "name": "Deutsch",
                            "icon_url": "/languages/de/icon"
                        },
                        "level": 5
                    }, {
                        "profile_id": 1,
                        "language": {
                            "id": "fr",
                            "name": "Französisch",
                            "icon_url": "/languages/fr/icon"
                        },
                        "level": 3
                    }]
                }
            })


if __name__ == "main":
    unittest.main()
